@@ -6,18 +6,31 @@
 Global state department.  Don't reload this module or everything breaks.
 """
 
-from __future__ import absolute_import, division, print_function
 
-import platform
 import sys
 import warnings
 
-from collections import OrderedDict
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    Optional,
+    Sequence,
+    Type,
+    cast,
+)
 
-from ._generic import BoundLogger
+from ._log_levels import make_filtering_bound_logger
 from ._loggers import PrintLoggerFactory
 from .dev import ConsoleRenderer, _has_colorama, set_exc_info
-from .processors import StackInfoRenderer, TimeStamper, format_exc_info
+from .processors import (
+    StackInfoRenderer,
+    TimeStamper,
+    add_log_level,
+    format_exc_info,
+)
+from .types import BindableLogger, Context, Processor, WrappedLogger
 
 
 """
@@ -25,37 +38,33 @@ from .processors import StackInfoRenderer, TimeStamper, format_exc_info
 
    Any changes to these defaults must be reflected in `getting-started`.
 """
-_BUILTIN_DEFAULT_PROCESSORS = [
+_BUILTIN_DEFAULT_PROCESSORS: Sequence[Processor] = [
+    add_log_level,
     StackInfoRenderer(),
     set_exc_info,
     format_exc_info,
     TimeStamper(fmt="%Y-%m-%d %H:%M.%S", utc=False),
-    ConsoleRenderer(colors=_has_colorama),
+    ConsoleRenderer(colors=_has_colorama and sys.stdout.isatty()),
 ]
-if (
-    sys.version_info[:2] >= (3, 6)
-    or platform.python_implementation() == "PyPy"
-):
-    # Python 3.6+ and PyPy have ordered dicts.
-    _BUILTIN_DEFAULT_CONTEXT_CLASS = dict
-else:
-    _BUILTIN_DEFAULT_CONTEXT_CLASS = OrderedDict
-_BUILTIN_DEFAULT_WRAPPER_CLASS = BoundLogger
+_BUILTIN_DEFAULT_CONTEXT_CLASS = cast(Type[Context], dict)
+_BUILTIN_DEFAULT_WRAPPER_CLASS = make_filtering_bound_logger(0)
 _BUILTIN_DEFAULT_LOGGER_FACTORY = PrintLoggerFactory()
 _BUILTIN_CACHE_LOGGER_ON_FIRST_USE = False
 
 
-class _Configuration(object):
+class _Configuration:
     """
     Global defaults.
     """
 
-    is_configured = False
-    default_processors = _BUILTIN_DEFAULT_PROCESSORS[:]
-    default_context_class = _BUILTIN_DEFAULT_CONTEXT_CLASS
-    default_wrapper_class = _BUILTIN_DEFAULT_WRAPPER_CLASS
-    logger_factory = _BUILTIN_DEFAULT_LOGGER_FACTORY
-    cache_logger_on_first_use = _BUILTIN_CACHE_LOGGER_ON_FIRST_USE
+    is_configured: bool = False
+    default_processors: Iterable[Processor] = _BUILTIN_DEFAULT_PROCESSORS[:]
+    default_context_class: Type[Context] = _BUILTIN_DEFAULT_CONTEXT_CLASS
+    default_wrapper_class: Any = _BUILTIN_DEFAULT_WRAPPER_CLASS
+    logger_factory: Callable[
+        ..., WrappedLogger
+    ] = _BUILTIN_DEFAULT_LOGGER_FACTORY
+    cache_logger_on_first_use: bool = _BUILTIN_CACHE_LOGGER_ON_FIRST_USE
 
 
 _CONFIG = _Configuration()
@@ -64,20 +73,18 @@ Global defaults used when arguments to `wrap_logger` are omitted.
 """
 
 
-def is_configured():
+def is_configured() -> bool:
     """
     Return whether ``structlog`` has been configured.
 
     If `False`, ``structlog`` is running with builtin defaults.
 
-    :rtype: bool
-
     .. versionadded: 18.1
     """
     return _CONFIG.is_configured
 
 
-def get_config():
+def get_config() -> Dict[str, Any]:
     """
     Get a dictionary with the current configuration.
 
@@ -85,8 +92,6 @@ def get_config():
 
        Changes to the returned dictionary do *not* affect ``structlog``.
 
-    :rtype: dict
-
     .. versionadded: 18.1
     """
     return {
@@ -98,7 +103,7 @@ def get_config():
     }
 
 
-def get_logger(*args, **initial_values):
+def get_logger(*args: Any, **initial_values: Any) -> Any:
     """
     Convenience function that returns a logger according to configuration.
 
@@ -113,7 +118,8 @@ def get_logger(*args, **initial_values):
     :param initial_values: Values that are used to pre-populate your contexts.
 
     :returns: A proxy that creates a correctly configured bound logger when
-        necessary.
+        necessary. The type of that bound logger depends on your configuration
+        and is `structlog.BoundLogger` by default.
 
     See `configuration` for details.
 
@@ -136,14 +142,14 @@ stick out like a sore thumb in frameworks like Twisted or Zope.
 
 
 def wrap_logger(
-    logger,
-    processors=None,
-    wrapper_class=None,
-    context_class=None,
-    cache_logger_on_first_use=None,
-    logger_factory_args=None,
-    **initial_values
-):
+    logger: WrappedLogger,
+    processors: Optional[Iterable[Processor]] = None,
+    wrapper_class: Optional[Type[BindableLogger]] = None,
+    context_class: Optional[Type[Context]] = None,
+    cache_logger_on_first_use: Optional[bool] = None,
+    logger_factory_args: Optional[Iterable[Any]] = None,
+    **initial_values: Any
+) -> Any:
     """
     Create a new bound logger for an arbitrary *logger*.
 
@@ -157,7 +163,7 @@ def wrap_logger(
     *is* possible.
 
     :param initial_values: Values that are used to pre-populate your contexts.
-    :param tuple logger_factory_args: Values that are passed unmodified as
+    :param logger_factory_args: Values that are passed unmodified as
         ``*logger_factory_args`` to the logger factory if not `None`.
 
     :returns: A proxy that creates a correctly configured bound logger when
@@ -180,12 +186,12 @@ def wrap_logger(
 
 
 def configure(
-    processors=None,
-    wrapper_class=None,
-    context_class=None,
-    logger_factory=None,
-    cache_logger_on_first_use=None,
-):
+    processors: Optional[Iterable[Processor]] = None,
+    wrapper_class: Optional[Type[BindableLogger]] = None,
+    context_class: Optional[Type[Context]] = None,
+    logger_factory: Optional[Callable[..., WrappedLogger]] = None,
+    cache_logger_on_first_use: Optional[bool] = None,
+) -> None:
     """
     Configures the **global** defaults.
 
@@ -200,14 +206,14 @@ def configure(
 
     Use `reset_defaults` to undo your changes.
 
-    :param list processors: List of processors.
-    :param type wrapper_class: Class to use for wrapping loggers instead of
+    :param processors: The processor chain. See :doc:`processors` for details.
+    :param wrapper_class: Class to use for wrapping loggers instead of
         `structlog.BoundLogger`.  See `standard-library`, :doc:`twisted`, and
         `custom-wrappers`.
-    :param type context_class: Class to be used for internal context keeping.
-    :param callable logger_factory: Factory to be called to create a new
+    :param context_class: Class to be used for internal context keeping.
+    :param logger_factory: Factory to be called to create a new
         logger that shall be wrapped.
-    :param bool cache_logger_on_first_use: `wrap_logger` doesn't return an
+    :param cache_logger_on_first_use: `wrap_logger` doesn't return an
         actual wrapped logger but a proxy that assembles one when it's first
         used.  If this option is set to `True`, this assembled logger is
         cached.  See `performance`.
@@ -216,6 +222,7 @@ def configure(
         *cache_logger_on_first_use*
     """
     _CONFIG.is_configured = True
+
     if processors is not None:
         _CONFIG.default_processors = processors
     if wrapper_class is not None:
@@ -223,27 +230,39 @@ def configure(
     if context_class is not None:
         _CONFIG.default_context_class = context_class
     if logger_factory is not None:
-        _CONFIG.logger_factory = logger_factory
+        _CONFIG.logger_factory = logger_factory  # type: ignore
     if cache_logger_on_first_use is not None:
         _CONFIG.cache_logger_on_first_use = cache_logger_on_first_use
 
 
-def configure_once(*args, **kw):
+def configure_once(
+    processors: Optional[Iterable[Processor]] = None,
+    wrapper_class: Optional[Type[BindableLogger]] = None,
+    context_class: Optional[Type[Context]] = None,
+    logger_factory: Optional[Callable[..., WrappedLogger]] = None,
+    cache_logger_on_first_use: Optional[bool] = None,
+) -> None:
     """
-    Configures iff structlog isn't configured yet.
+    Configures if structlog isn't configured yet.
 
-    It does *not* matter whether is was configured using `configure` or
+    It does *not* matter whether it was configured using `configure` or
     `configure_once` before.
 
     Raises a `RuntimeWarning` if repeated configuration is attempted.
     """
     if not _CONFIG.is_configured:
-        configure(*args, **kw)
+        configure(
+            processors=processors,
+            wrapper_class=wrapper_class,
+            context_class=context_class,
+            logger_factory=logger_factory,
+            cache_logger_on_first_use=cache_logger_on_first_use,
+        )
     else:
         warnings.warn("Repeated configuration attempted.", RuntimeWarning)
 
 
-def reset_defaults():
+def reset_defaults() -> None:
     """
     Resets global default values to builtin defaults.
 
@@ -253,11 +272,11 @@ def reset_defaults():
     _CONFIG.default_processors = _BUILTIN_DEFAULT_PROCESSORS[:]
     _CONFIG.default_wrapper_class = _BUILTIN_DEFAULT_WRAPPER_CLASS
     _CONFIG.default_context_class = _BUILTIN_DEFAULT_CONTEXT_CLASS
-    _CONFIG.logger_factory = _BUILTIN_DEFAULT_LOGGER_FACTORY
+    _CONFIG.logger_factory = _BUILTIN_DEFAULT_LOGGER_FACTORY  # type: ignore
     _CONFIG.cache_logger_on_first_use = _BUILTIN_CACHE_LOGGER_ON_FIRST_USE
 
 
-class BoundLoggerLazyProxy(object):
+class BoundLoggerLazyProxy:
     """
     Instantiates a ``BoundLogger`` on first usage.
 
@@ -275,14 +294,14 @@ class BoundLoggerLazyProxy(object):
 
     def __init__(
         self,
-        logger,
-        wrapper_class=None,
-        processors=None,
-        context_class=None,
-        cache_logger_on_first_use=None,
-        initial_values=None,
-        logger_factory_args=None,
-    ):
+        logger: WrappedLogger,
+        wrapper_class: Optional[Type[BindableLogger]] = None,
+        processors: Optional[Iterable[Processor]] = None,
+        context_class: Optional[Type[Context]] = None,
+        cache_logger_on_first_use: Optional[bool] = None,
+        initial_values: Optional[Dict[str, Any]] = None,
+        logger_factory_args: Any = None,
+    ) -> None:
         self._logger = logger
         self._wrapper_class = wrapper_class
         self._processors = processors
@@ -291,7 +310,7 @@ class BoundLoggerLazyProxy(object):
         self._initial_values = initial_values or {}
         self._logger_factory_args = logger_factory_args or ()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return (
             "<BoundLoggerLazyProxy(logger={0._logger!r}, wrapper_class="
             "{0._wrapper_class!r}, processors={0._processors!r}, "
@@ -300,7 +319,7 @@ class BoundLoggerLazyProxy(object):
             "logger_factory_args={0._logger_factory_args!r})>".format(self)
         )
 
-    def bind(self, **new_values):
+    def bind(self, **new_values: Any) -> BindableLogger:
         """
         Assemble a new BoundLogger from arguments and configuration.
         """
@@ -308,7 +327,7 @@ class BoundLoggerLazyProxy(object):
             ctx = self._context_class(self._initial_values)
         else:
             ctx = _CONFIG.default_context_class(self._initial_values)
-        cls = self._wrapper_class or _CONFIG.default_wrapper_class
+
         _logger = self._logger
         if not _logger:
             _logger = _CONFIG.logger_factory(*self._logger_factory_args)
@@ -317,9 +336,13 @@ class BoundLoggerLazyProxy(object):
             procs = _CONFIG.default_processors
         else:
             procs = self._processors
-        logger = cls(_logger, processors=procs, context=ctx)
 
-        def finalized_bind(**new_values):
+        cls = self._wrapper_class or _CONFIG.default_wrapper_class
+        # Looks like Protocols ignore definitions of __init__ so we have to
+        # silence Mypy here.
+        logger = cls(_logger, processors=procs, context=ctx)  # type: ignore
+
+        def finalized_bind(**new_values: Any) -> BindableLogger:
             """
             Use cached assembled logger to bind potentially new values.
             """
@@ -332,10 +355,11 @@ class BoundLoggerLazyProxy(object):
             self._cache_logger_on_first_use is None
             and _CONFIG.cache_logger_on_first_use is True
         ):
-            self.bind = finalized_bind
+            self.bind = finalized_bind  # type: ignore
+
         return finalized_bind(**new_values)
 
-    def unbind(self, *keys):
+    def unbind(self, *keys: str) -> BindableLogger:
         """
         Same as bind, except unbind *keys* first.
 
@@ -343,7 +367,10 @@ class BoundLoggerLazyProxy(object):
         """
         return self.bind().unbind(*keys)
 
-    def new(self, **new_values):
+    def try_unbind(self, *keys: str) -> BindableLogger:
+        return self.bind().try_unbind(*keys)
+
+    def new(self, **new_values: Any) -> BindableLogger:
         """
         Clear context, then bind.
         """
@@ -351,26 +378,30 @@ class BoundLoggerLazyProxy(object):
             self._context_class().clear()
         else:
             _CONFIG.default_context_class().clear()
+
         bl = self.bind(**new_values)
+
         return bl
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> Any:
         """
         If a logging method if called on a lazy proxy, we have to create an
         ephemeral BoundLogger first.
         """
         if name == "__isabstractmethod__":
             raise AttributeError
+
         bl = self.bind()
+
         return getattr(bl, name)
 
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         """
         Our __getattr__ magic makes this necessary.
         """
         return self.__dict__
 
-    def __setstate__(self, state):
+    def __setstate__(self, state: Dict[str, Any]) -> None:
         """
         Our __getattr__ magic makes this necessary.
         """
